"""Top-level package for NEP29 Calculator."""

__author__ = 'Mark Harfouche'
__email__ = 'mark.harfouche@gmail.com'

from .nep29 import nep29_versions
from ._version import __version__

__all__ = [
    'nep29_versions',
    '__version__',
]



import nep29
from nep29 import nep29_versions
import pytest
from unittest.mock import patch, Mock
from datetime import datetime
from packaging.version import Version


def test_project_import():
    pass


@pytest.mark.parametrize('vstring,expected', [
    ("1!2.3.4", (1, 2, 3)),
    ("1.2.3", (0, 1, 2)),
])
def test_major_minor_version(vstring, expected):
    from packaging.version import Version
    v = Version(vstring)
    assert nep29.nep29.major_minor_version(v) == expected


def _mock_release_info(release_info):
    # returns a mock for the requests.Response object
    def _make_release_entry(version, date, packagetype='sdist'):
        return version, [{
            'packagetype': packagetype,
            'upload_time_iso_8601': date + 'T00:00:00.000000Z'
        }]

    package = {'releases': dict(_make_release_entry(*info)
                                for info in release_info)}
    request = Mock()
    request.json = Mock(return_value=package)
    return request


@patch.object(nep29.nep29, 'datetime', Mock(wraps=datetime))
def _nep29_versions_test(request_mock, expected, *args, **kwargs):
    # core code to run tests of nep29_versions from a mock for the
    # requests.Response object as a list of expected version strings;
    # additional args/kwargs passed to nep29_versions
    nep29.nep29.datetime.utcnow.return_value = datetime(2021, 1, 1)
    with patch('nep29.nep29.requests.get', Mock(return_value=request_mock)):
        results = nep29_versions('foo', *args, **kwargs)

    found_versions = set(r[0] for r in results)
    assert found_versions == set(Version(v) for v in expected)


@pytest.mark.parametrize('releases,expected', [
    # NOTE: for these tests we patch so that today is 2021-01-01, midnight.
    # All mock releases are at midnight GMT.
    (
        # default: include up to 3 minor versions if released long ago
        _mock_release_info([('2.0.0', '2015-01-01'),
                            ('1.2.0', '2014-01-01'),
                            ('1.1.0', '2013-01-01'),
                            ('1.0.0', '2012-01-01')]),
        ['2.0.0', '1.2.0', '1.1.0']
    ),
    (
        # default: include all minor versions first released in 24 months
        _mock_release_info([('1.9.0', '2020-12-01'),
                            ('1.8.0', '2020-09-01'),
                            ('1.7.0', '2020-06-01'),
                            ('1.6.0', '2020-01-01'),
                            ('1.5.0', '2019-06-01'),
                            ('1.4.0', '2018-12-01')]),
        ['1.9.0', '1.8.0', '1.7.0', '1.6.0', '1.5.0']
    ),
    (
        # default: release candidates should not be included
        _mock_release_info([('2.0.0.rc0', '2020-12-01'),
                            ('1.3.0', '2020-11-01'),
                            ('1.2.0', '2020-10-01'),
                            ('1.1.0', '2020-09-01')]),
        ['1.3.0', '1.2.0', '1.1.0']
    ),
])
def test_nep29_versions(releases, expected):
    _nep29_versions_test(releases, expected)


def test_nep29_versions_allow_rc():
    releases = _mock_release_info([('2.0.0.rc0', '2020-12-01'),
                                   ('1.3.0', '2020-11-01'),
                                   ('1.2.0', '2020-10-01'),
                                   ('1.1.0', '2020-09-01')])
    expected = ['2.0.0.rc0', '1.3.0', '1.2.0', '1.1.0']
    _nep29_versions_test(releases, expected, skip_rc=False)


def test_pandas():
    # Pre-release information can be included as letters
    # not seperated by a '.' from the version number
    # the pandas package uses this notation.
    nep29_versions('pandas')



#!/usr/bin/env python3
from setuptools import setup, find_packages


def get_version_and_cmdclass(pkg_path):
    """Load version.py module without importing the whole package.

    Template code from miniver
    """
    import os
    from importlib.util import module_from_spec, spec_from_file_location

    version_py = os.path.join(pkg_path, "_version.py")
    spec = spec_from_file_location("version", version_py)
    module = module_from_spec(spec)
    spec.loader.exec_module(module)
    return module.__version__, module.get_cmdclass(pkg_path)


with open('README.md') as readme_file:
    readme = readme_file.read()

with open('HISTORY.md') as history_file:
    history = history_file.read()

requirements = ['requests', 'prettytable', 'packaging', ]

test_requirements = ['pytest', ]

version, cmdclass = get_version_and_cmdclass(r"nep29")

setup(
    author="Mark Harfouche",
    author_email='mark.harfouche@gmail.com',
    classifiers=[
        'Development Status :: 2 - Pre-Alpha',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: BSD License',
        'Natural Language :: English',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.6',
        'Programming Language :: Python :: 3.7',
        'Programming Language :: Python :: 3.8',
        'Programming Language :: Python :: 3.9',
    ],
    python_requires='>=3.6',
    description="NEP29 Calculator tools",
    install_requires=requirements,
    license="BSD license",
    long_description=readme + '\n\n' + history,
    long_description_content_type='text/markdown',
    include_package_data=True,
    keywords='nep29',
    name='nep29',
    packages=find_packages(),
    tests_require=test_requirements,
    url='https://github.com/hmaarrfk/nep29',
    entry_points={
        'console_scripts': [
            'nep29=nep29.nep29:main'
        ],
    },
    version=version,
    cmdclass=cmdclass,
    zip_safe=False,
)
